""" pyWiewRawFilter.py --- class used by pyWiew to perform image filtering/effects using raw image data
Copyright 2002-2003 Anand B Pillai <abpillai@lycos.com> """

import array
import thread, threading
from wxPython.wx import *
from pyWiewGlobals import *

class pyWiewRawFilter(threading.Thread):
	""" Filter image data using raw image data 
	This class expects a wxImage instance and uses a worker thread """

	def __init__(self, name, frame, img, filter, args):
		
		threading.Thread.__init__(self, None, None, name)
		self._img = img.Copy()
		self._imgout = self._img.Copy()
		self._arr = array.array('B', img.GetData())
		self._imgdatalist = self._arr.tolist()
		self._manip=true #means array level manip was done
		self._frame=frame
		self._filter = filter
		self._args = args
		self._worker=None
		self._contf = 1.0
		self._brigf = 1.0
		 
	def run(self):
		wait=wxBusyCursor()
		#self._worker=threading.Thread(None, 'FilterOp', self.Filter, ())
		self.Filter()
		self.FilteredImage()

	def GetImage(self):
		return self._imgout
	
	def Filter(self):
		"Filter main function"
		
		code = self._filter
		data = self._args
		
		if code == "negate":
			#Threading seems faster when compared to mapping
			#or making the function-call directly
			self.Negate()
		elif code == "bluechn":
			self.ColorChannel('blue')
		elif code == "redchn":
			self.ColorChannel('red')
		elif code == "greenchn":
			self.ColorChannel('green')
		elif code == "switch":
			offset=data[0]
			self.ColorSwitch(offset)
		elif code == "reverse":
			self.ReverseImage()
		elif code == "depth":
			self.ChangeDepth(data[0])
		elif code == 'grayscale':
			self.GrayScale()
		elif code == 'contrast':
			self._contf=data[0]
			self.Contrast()
		elif code == 'brighten':
			self._brigf = data[0]
			self.Brighten()

	def FilteredImage(self):
		if self._manip == true:
			arr = array.array('B', self._imgdatalist)
			self._imgout.SetData(arr.tostring())

	def Negate(self,bit=0):
		#this form uses map approach (commented)
		#self._imgdatalist = map(lambda x: 255 - x, self._imgdatalist)
		#this form uses python's list comprehension approach
		self._imgdatalist = [255 -x for x in self._imgdatalist]
		
	def ReverseImage(self):
		self._imgdatalist.reverse()

	def GrayScale(self):
			
		index=0
		while index<len(self._imgdatalist):
			r=self._imgdatalist[index]
			g=self._imgdatalist[index+1]
			b=self._imgdatalist[index+2]
			gray=int(0.56*g	 + 0.33*r + 0.11*b)
			rg=gray
			gg=gray
			bg=gray
			self._imgdatalist[index]=rg
			self._imgdatalist[index+1]=gg
			self._imgdatalist[index+2]=bg
			
			index += 3

	def Brighten(self):
		"Do a brightness enhancement"
		
		self._imgdatalist = map(self.BrightenOp, self._imgdatalist)
		 
	def Contrast(self):
		 "Do a contrast enhancement"
		 self._imgdatalist = map(self.ContrastOp, self._imgdatalist)
		 
	def BrightenOp(self, pixel):

		#increase the pixel brightness
		pixel = int(pixel * self._brigf)
		if pixel > 255:
			pixel = 255

		return pixel
	
	def ContrastOp(self, pixel):
		 
		if pixel < 128:
			pixel = int(pixel / self._contf)
		else:
			pixel = int(pixel * self._contf)
			if pixel > 255:
				pixel = 255

		return pixel						
								 
	def ColorChannel(self, colorname):
		"Returns the r,g or b color channel"
		
		index = 0

		if colorname == "red":
			#self._imgdatalist = [x for x in self._imgdatalist if not x % 3]
			while index < len(self._imgdatalist):
				self._imgdatalist[index+1], self._imgdatalist[index+2] = 0, 0
				index = index + 3

		elif colorname == "blue":
			while index < len(self._imgdatalist):
				self._imgdatalist[index], self._imgdatalist[index+1] = 0, 0
				index = index + 3

		elif colorname == "green":
			while index < len(self._imgdatalist):
				self._imgdatalist[index], self._imgdatalist[index+2] = 0, 0
				index = index + 3

	def ColorSwitch(self,offset):
		""" Switch colors (rgb->gbr rgb->brg, rgb->bgr, rgb->rbg) etc """
		#offset values
		#1 -> right offset 1
		#2 -> right offset 2
		#-1 -> reverse
		#23 -> flip 2 & 3 posn
		#12 -> flip 1 & 2 posn
		index = 0
		
		while index<len(self._imgdatalist):
			#rgb=self._imgdatalist[index:index+2]
			r=self._imgdatalist[index]
			g=self._imgdatalist[index+1]
			b=self._imgdatalist[index+2]
			if offset == 1: #rgb->gbr
				self._imgdatalist[index],self._imgdatalist[index+1],self._imgdatalist[index+2] = g,b,r
			elif offset == 2: #rgb->brg
				self._imgdatalist[index],self._imgdatalist[index+1],self._imgdatalist[index+2] = b,r,g
			elif offset == -1: #rgb->bgr
				self._imgdatalist[index],self._imgdatalist[index+1],self._imgdatalist[index+2] = b,g,r
			elif offset == 23: #rgb->rbg
				self._imgdatalist[index],self._imgdatalist[index+1],self._imgdatalist[index+2] = r,b,g
			elif offset == 12: #rgb->grb
				self._imgdatalist[index],self._imgdatalist[index+1],self._imgdatalist[index+2] = g,r,b

			index += 3

	def ChangeDepth(self,depth): #code not working!
		"""Changes the color depth of the bitmap"""
		
		#here we deal directly with the bitmap
		#so we set nomanip to true
		self._manip=false
		bmap=self._img.ConvertToBitmap()
		bmap=bmap.Create(self._img.GetData(),wxBITMAP_TYPE_BMP,self._img.GetWidth(),self._img.GetHeight(),depth)
		#bmap.SetDepth(depth)
		#bmap.SetWidth(self._img.GetWidth())
		#bmap.SetHeight(self._img.GetHeight())
		self._imgout=wxImageFromBitmap(bmap)

